// Copyright (c) 2019, Christoph Peters
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the Karlsruhe Institute of Technology nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include "SphereLight.h"
#include "Falcor.h"
#include <sstream>

bool SphereLight::renderUI(Falcor::Gui* pGui,uint32_t iLight){
	std::ostringstream stringStream;
	stringStream<<iLight;
	std::string suffix=stringStream.str();
	std::string groupName="Sphere light "+suffix;
	if(pGui->beginGroup(groupName)){
		pGui->addFloat3Var(("Center "+suffix).c_str(),posW);
		// Editing the power (in deca Watt) is more convenient than editing the 
		// radiance
		bool updateRadiance=false;
		const float conversionFactor=0.1f*4.0f*static_cast<float>(M_PI*M_PI);
		glm::vec3 power=(conversionFactor*radius*radius)*surfaceRadiance;
		updateRadiance|=(pGui->addFloatVar(("Radius "+suffix).c_str(),radius,0.001f)!=0);
		updateRadiance|=(pGui->addRgbColor(("Power "+suffix).c_str(),power)!=0);
		if(updateRadiance && radius>0.0f){
			surfaceRadiance=power/(conversionFactor*radius*radius);
		}
		bool result=pGui->addButton("Delete");
		pGui->endGroup();
		return result;
	}
	return false;
}
