// Copyright (c) 2019, Christoph Peters
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the Karlsruhe Institute of Technology nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#pragma once
#include "glm/vec3.hpp"

namespace Falcor{
	class Gui;
}

/*! This class represents a spherical light source that emits light 
	isotropically in all directions. By design, it matches the layout of the 
	corresponding structure in the shader.*/
struct SphereLight{
	/*! Constructs with the given attributes.
	   \sa posW,radius,surfaceRadiance*/
	SphereLight(glm::vec3 worldSpaceCenterPosition,float sphereRadius,glm::vec3 uniformSurfaceRadiance):
	posW(worldSpaceCenterPosition),
	radius(sphereRadius),
	surfaceRadiance(uniformSurfaceRadiance){
	}

	/*! Renders a user interface that enables full control over position, 
		radius and radiance of this light source.
		\return true iff the button to delete the light source has been 
				clicked.*/
	bool renderUI(Falcor::Gui* pGui,uint32_t iLight);

	/*! The center of the sphere in world space coordinates.*/
	glm::vec3 posW;
	/*! The radius of the sphere in world space coordinates.*/
	float radius;
	/*! Each point on the surface of the sphere emits light isotropically in 
		all directions. This vector gives the light radiance in linear RGB.*/
	glm::vec3 surfaceRadiance;
};
